import * as THREE from 'three';
import { useContextKey } from '@kevisual/store/context';
import { CSS3DRenderer, CSS3DObject } from 'three/examples/jsm/renderers/CSS3DRenderer';
import { OrbitControls } from 'three/examples/jsm/controls/OrbitControls';
export const renderer = useContextKey('renderer', () => {
  const renderer = new THREE.WebGLRenderer();
  renderer.setSize(window.innerWidth, window.innerHeight);
  return renderer;
});

export const mainScene = useContextKey('mainScene', () => {
  const scene = new THREE.Scene();
  return scene;
});

export const cssRenderer = useContextKey('cssRenderer', () => {
  const renderer = new CSS3DRenderer();
  renderer.setSize(window.innerWidth, window.innerHeight);
  return renderer;
});
export const cssScene = useContextKey('cssScene', () => {
  const scene = new THREE.Scene();
  return scene;
});

export const mainCamera = useContextKey('mainCamera', () => {
  const camera = new THREE.PerspectiveCamera(75, window.innerWidth / window.innerHeight, 0.1, 10000);
  return camera;
});
// export const mainControls = useContextKey('mainControls', () => {
//   const controls = new OrbitControls(mainCamera, renderer.domElement);
//   return controls;
// });

export const render = ({ root, width, height }: { root: HTMLElement; width: number; height: number }) => {
  const camera = mainCamera;

  root.appendChild(renderer.domElement);
  root.appendChild(cssRenderer.domElement);

  renderer.render(mainScene, camera);

  camera.position.z = 453;

  // 设置CSS渲染器的样式
  cssRenderer.domElement.style.position = 'absolute';
  cssRenderer.domElement.style.top = '0';
  cssRenderer.domElement.style.pointerEvents = 'auto';
  const controls = new OrbitControls(camera, cssRenderer.domElement);

  function animate() {
    renderer.render(mainScene, camera);
    // cssRenderer.render(cssScene, camera);
    controls.update();
  }
  renderer.setAnimationLoop(animate);
  return { renderer, cssRenderer };
};
// add a cube to the scene

export const addCube = () => {
  const geometry = new THREE.BoxGeometry(1000, 1000, 1000);
  const material = new THREE.MeshBasicMaterial({ color: 0x00ff00 });
  const cube = new THREE.Mesh(geometry, material);
  setInterval(() => {
    cube.rotation.x += 0.01;
    cube.rotation.y += 0.01;
  }, 16);
  mainScene.add(cube);
};

export const addDivDocument = () => {
  const div = document.createElement('div');
  // div.style.width = '1920px';
  // div.style.height = '1080px';
  div.style.width = window.innerWidth + 'px';
  div.style.height = window.innerHeight + 'px';
  div.style.backgroundColor = 'red';
  div.style.overflow = 'scroll';
  div.innerHTML = 'Hello<br>World';
  for (let i = 0; i < 2000; i++) {
    div.innerHTML += 'Hello<br>World';
  }
  // document.body.appendChild(div);
  // 挂载到mainScene
  const divObject = new CSS3DObject(div);
  divObject.position.set(0, 0, 0);
  divObject.scale.set(1, 1, 1);
  divObject.rotation.set(0, 0, 0);
  divObject.visible = true;
  let count = 1;
  div.addEventListener('click', () => {
    console.log('click');
    count++;
  });

  divObject.userData = {
    name: 'divObject',
  };
  cssScene.add(divObject);

  // setInterval(() => {
  //   div.style.transform = `translate(${Math.random() * 100}px, ${Math.random() * 100}px)`;
  // }, 16);
};

export const addDivDocument2 = () => {
  const div = document.createElement('div');
  div.style.width = window.innerWidth + 'px';
  div.style.height = window.innerHeight + 'px';
  div.style.backgroundColor = 'blue';
  div.style.overflow = 'scroll';
  div.innerHTML = 'Hello<br>World';
  for (let i = 0; i < 2000; i++) {
    div.innerHTML += 'Hello<br>World';
  }
  document.body.appendChild(div);
  const divObject = new CSS3DObject(div);
  divObject.position.set(0, -window.innerHeight, 0);
  divObject.scale.set(1, 1, 1);
  divObject.rotation.set(0, 0, 0);
  divObject.visible = true;
  cssScene.add(divObject);
};
